from rest_framework.permissions import SAFE_METHODS, BasePermission


class IsStaffOrReadOnly(BasePermission):
    """
    The request is authenticated as a staff member, or is a read-only request.
    """

    def has_permission(self, request, view):
        if request.method in SAFE_METHODS:
            return True
        return request.user and request.user.is_staff



from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.permissions import IsAdminUser

from evo_django_kits.entities.evo_response import EvoResponse
from evo_django_kits.entities.serializers.bulk_delete_serializer import BulkDeleteSerializer


class BaseViewSet(viewsets.ModelViewSet):
    response = EvoResponse

    def initial(self, request, *args, **kwargs):
        super().initial(request, *args, **kwargs)

    def get_serializer_context(self):
        context = super().get_serializer_context()
        context.update({"user": self.request.user})
        return context

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        return self.response(
            data=serializer.data,
            status=201,
            message="Created Successfully",
            headers=headers,
        )

    def update(self, request, *args, **kwargs):
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data, partial=kwargs.get("partial", False))
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return self.response(data=serializer.data, status=200, message="Updated Successfully")

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        self.perform_destroy(instance)
        return self.response(status=204, message="Deleted Successfully")

    @action(
        detail=False,
        methods=["DELETE"],
        url_name="bulk-delete",
        serializer_class=BulkDeleteSerializer,
        permission_classes=[IsAdminUser],
    )
    def bulk_delete(self, request):
        """
        For each list end point have endpoint to bulk delete with param ids
        example: /users/bulk_delete/?ids=21,22
        """
        ids = request.query_params.get("ids", None)
        ids = [int(_id) for _id in ids.split(",")] if ids else []
        data = {"ids": ids}
        serializer = self.serializer_class(data=data)
        serializer.is_valid(raise_exception=True)
        ids = serializer.validated_data["ids"]
        self.queryset.filter(id__in=ids).delete()
        return self.response(status=204, data=[ids], message=f"Delete {len(ids)} successful")



import logging


class EvoLogger(logging.Logger):
    def __init__(self, name="evo-django-kits", level=logging.DEBUG):
        super().__init__(name, level)
        self.setLevel(level)
        self.addHandler(logging.StreamHandler())
        self.addHandler(logging.FileHandler("evo.log"))
        self.propagate = False

    def log(self, level, msg, *args, **kwargs):
        super().log(level, msg, *args, **kwargs)



from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response


class EvoPageNumberPagination(PageNumberPagination):

    page_size = 10
    page_size_query_param = "page_size"
    max_page_size = 100
    last_page_strings = ("last",)

    def get_paginated_response(self, data):
        return Response(
            {
                "count": self.page.paginator.count,
                "next": self.get_next_link(),
                "previous": self.get_previous_link(),
                "page": self.page.number,
                "page_size": self.page_size,
                "results": data,
            }
        )



from rest_framework.response import Response


class EvoResponse(Response):
    def __init__(
        self,
        message=None,
        data=None,
        status=None,
        headers=None,
    ):
        status = status or 200

        response = {"message": message}
        if data is not None:
            response["data"] = data

        super().__init__(response, status=status, headers=headers)
